/*
** Module   : MsSqlHelper.java
** Abstract : Helper class which initializes MSSQL 2012 database (& 2008 ?)
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- --------------------------- Description ---------------------------
** 001 OM  20140307   @37516 Created initial version. Helper class which provides utilities for
**                           Microsoft SQL server 2012.
*/

package com.goldencode.p2j.persist.embed;

import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.persist.pl.BuiltIns;
import java.lang.reflect.Method;
import java.util.*;

/**
 * Helper class which provides MSSQL-specific services: TBA
 */
public final class MsSqlHelper
{
   /**
    * Private constructor.  All access to this class is via static methods.
    */
   private MsSqlHelper()
   {
   }

   /**
    * Register the function aliases with the HQL preprocessor, for the permanent database.
    *
    * @param   database
    *          Database to prepare.
    */
   public static void preparePermanentDatabase(Database database)
   {
      getFunctionAliases(database, BuiltIns.getPublicFunctions(), true, true);
      getFunctionAliases(database, BuiltIns.getSupportFunctions(), false, true);
   }

   /**
    * Get a list of database prepare DDL statements.
    *
    * @param   database
    *          Database to prepare.
    *
    * @return  the list of DDL statements.
    */
   public static List<String> getPrepareStatements(Database database)
   {
      // Create built-in function aliases.
      return getFunctionAliases(database, BuiltIns.getPublicFunctions(), true, false);
   }

   /**
    * Get function aliases for the given list of static methods, optionally making the alias names
    * unique by appending parameter type decorations.
    *
    * @param   database
    *          Database in which to create aliases.
    * @param   funcs
    *          List of static methods for which function aliases should be created.
    * @param   overload
    *          <code>true</code> to create unique function names for each overloaded method.
    * @param   register
    *          <code>true</code> if the function should be registered with
    *          the {@link com.goldencode.p2j.persist.HQLPreprocessor}.
    */
   private static List<String> getFunctionAliases(Database database,
                                                  List<Method> funcs,
                                                  boolean overload,
                                                  boolean register)
   {
      List<String> ddl = new ArrayList<>();

      for (Method method : funcs)
      {
         String uniqueName = method.getName();
         StringBuilder functionDecoration = new StringBuilder("_");
         StringBuilder paramList = new StringBuilder();

         Class<?>[] signature = method.getParameterTypes();
         for (int i = 0; i < signature.length; i++)
         {
            if (i > 0)
            {
               paramList.append(", ");
            }
            String paramType = signature[i].getName();
            paramList.append("@param").append(i + 1);
            paramList.append(" ").append(getSqlType(paramType));
            functionDecoration.append(getSqlDecoration(paramType));
         }

         if (overload)
         {
            uniqueName += functionDecoration.toString();
         }

         StringBuilder buf = new StringBuilder("\ncreate function [dbo].[");
         buf.append(uniqueName).append(']');
         buf.append("\n    (").append(paramList.toString()).append(")");
         buf.append("\n    returns ").append(getSqlType(method.getReturnType().getName()));
         buf.append("\nas external name [P2JWrappers].[");
         buf.append(method.getDeclaringClass().getSimpleName());
         buf.append("].[").append(uniqueName).append("];");

         ddl.add(buf.toString());
         ddl.add("go;");   // only one CREATE FUNCTION in a batch is allowed

         if (overload && register)
         {
            // drop the [ / ] in order to pass through Hibernate
            HQLPreprocessor.registerFunction(database, "dbo." + uniqueName, method);
         }
      }

      return ddl;
   }

   /**
    * Returns the MS SQL Server datatype associated with the java type.
    *
    * @param   javaType
    *          The java type (full name).
    *
    * @return  The MS SQL Server datatype mapped by <code>javaType</code>.
    *          If <code>javaType</code> is invalid, <code>null</code> is returned.
    */
   private static String getSqlType(String javaType)
   {
      switch (javaType)
      {
         case "java.lang.Boolean":
         case "boolean":
            return "bit";

         case "java.lang.Integer":
            return "int";

         case "java.lang.Long":
            return "bigint";

         case "java.lang.String":
            return "nvarchar(4000)";

         case "java.math.BigDecimal":
            return "double precision";

         case "java.sql.Date":
            return "date";

         case "java.sql.Timestamp":
            return "datetime2";
      }
      return null;
   }

   /**
    * Returns the parameter decoration for fixing the function overloading constraint in
    * MS SQL Server.
    *
    * @param   javaType
    *          The java type (full name).
    *
    * @return  a character for decorating the SQL function.
    */
   private static char getSqlDecoration(String javaType)
   {
      switch (javaType)
      {
         case "java.lang.Boolean":
         case "boolean":
            return 'b';

         case "java.lang.Integer":
            return 'i';

         case "java.lang.Long":
            return 'l';

         case "java.lang.String":
            return 's';

         case "java.math.BigDecimal":
            return 'r';

         case "java.sql.Date":
            return 'd';

         case "java.sql.Timestamp":
            return 't';
      }
      return ' ';
   }
}
