/*
** Module   : Operators.cs
** Abstract : Delegates the CLR decorated methods to overloaded operators from assembly converted
** from java p2jpl.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --DateTime-- -------------------------------- Description --------------------------------
** 001 OM  20140314 Created initial version with full support of existing functions from
**                  Operators.java at this time.
*/

using System;

/**
 * The role of this class is to resolve the SQL Server method overload constraint and simplify the
 * name of the functions.
 * <p/>
 * Each method of this class has as correspondent method in the original
 * <code>com.goldencode.p2j.persist.pl.Operators</code> that will call with the parameters
 * converted for wrapped in IKVM converted datatypes. The method names of the original class is
 * decorated with as list of characters, one for each parameter, each such character represents
 * the type of the parameters in the original method signature. This way, there are no overloaded
 * methods so SQL Server cannot complain about this.
 * <p/>
 * The decorations are as follows: <ul> <li>s - for character / string / (string / string) /
 * NVARCHAR(4000) <li>b - for logical / (boolean / Boolean) / (bool / Boolean) / BIT <li>i - for
 * integer / (int / Int32) / (int / Int32) / INT <li>l - for int64 / (long / Int64) / (long /
 * Int64) / BIGINT <li>r - for decimal / (double / Double) / (double / Double) / (DOUBLE PRECISION
 * / DECIMAL(38)) <li>d - for date / DateTime / DateTime / DATE <li>t - for datetime / DateTime /
 * DateTime / DATETIME2 </ul>
 * <p/>
 * For each function an UDF will be defined in SQL having the same name as the method of this
 * class.
 * <p/>
 * Note that, if the original interface of the <code>Operators</code> is changed (new methods are
 * added or existing ones change their signature) the current class must be adjusted to fit those
 * changes (add/remove method or signature change).
 */
public class Operators
{
   /**
    * Private constructor. This class is never instantiated. All access is done using static
    * public methods.
    */
   private Operators()
   {
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toBigDecimal(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toBigDecimal(a),
                                                        Common.toInteger(b)).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toInteger(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toInteger(a), Common.toInteger(b)
      ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(a, b).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.eq(Common.toTimestamp(a),
                                                        Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(a, b).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toInteger(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toBigDecimal(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toBigDecimal(a),
                                                        Common.toInteger(b)).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isNotEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toInteger(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toTimestamp(a),
                                                        Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test two values for inequality.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean ne_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.ne(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(a, b).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toInteger(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toBigDecimal(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toBigDecimal(a),
                                                        Common.toInteger(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toInteger(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toTimestamp(a),
                                                        Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gt_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gt(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(a, b).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toInteger(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toBigDecimal(a),
                                                         Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toBigDecimal(a),
                                                         Common.toInteger(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toInteger(a),
                                                         Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toTimestamp(a),
                                                         Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &gt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean gte_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.gte(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(a, b).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toInteger(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toBigDecimal(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toBigDecimal(a),
                                                        Common.toInteger(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThan(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toInteger(a),
                                                        Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toTimestamp(a),
                                                        Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt; <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lt_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lt(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_bb(Boolean a, Boolean b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toBoolean(a), Common.toBoolean(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_ss(string a, string b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(a, b).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toLong(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toLong(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toInteger(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toInteger(a), Common.toInteger(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toBigDecimal(a),
                                                         Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toBigDecimal(a), Common.toLong(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toBigDecimal(a),
                                                         Common.toInteger(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toLong(a), Common.toBigDecimal(b))
            .booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isLessThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toInteger(a),
                                                         Common.toBigDecimal(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_tt(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toTimestamp(a),
                                                         Common.toTimestamp(b)).booleanValue();
   }

   /**
    * Test whether <code>a</code> &lt;= <code>b</code>.
    *
    * @see com.goldencode.p2j.util.CompareOps#isGreaterThanOrEqual(com.goldencode.p2j.util.BaseDataType,
    * com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean lte_dd(DateTime a, DateTime b)
   {
      return com.goldencode.p2j.persist.pl.Operators.lte(Common.toDate(a), Common.toDate(b))
            .booleanValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 plus_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toLong(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 plus_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toLong(a), Common.toInteger(b))
            .longValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 plus_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toInteger(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 plus_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toInteger(a),
                                                          Common.toInteger(b)).longValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double plus_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toBigDecimal(a),
                                                          Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double plus_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toBigDecimal(a),
                                                          Common.toLong(b)).doubleValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double plus_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toBigDecimal(a),
                                                          Common.toInteger(b)).doubleValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double plus_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toLong(a),
                                                          Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Add two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#plus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double plus_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.plus(Common.toInteger(a),
                                                          Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_dl(DateTime value, Int64 days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toDate(value),
                                                                          Common.toLong(days)));
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_di(DateTime value, Int32 days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toDate(value),
                                                                          Common.toInteger(days)));
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_dr(DateTime value, Double days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toDate(value),
                                                                          Common.toBigDecimal(days)));
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_ld(Int64 days, DateTime value)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toLong(days),
                                                                          Common.toDate(value)));
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_id(Int32 days, DateTime value)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toInteger(days),
                                                                          Common.toDate(value)));
   }

   /**
    * Add a number of days to a date.
    *
    * @see com.goldencode.p2j.util.date#plusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plus_rd(Double days, DateTime value)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.plus(Common.toBigDecimal(days),
                                                                          Common.toDate(value)));
   }

   /**
    * Add a number of milliseconds to a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#plusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plusMillis_rt(Double millis, DateTime value)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.plusMillis(Common.toBigDecimal(
            millis), Common.toTimestamp(value)));
   }

   /**
    * Add a number of milliseconds to a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#plusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plusMillis_tr(DateTime value, Double millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.plusMillis(Common.toTimestamp(
            value), Common.toBigDecimal(millis)));
   }

   /**
    * Add a number of milliseconds to a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#plusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plusMillis_tl(DateTime value, Int64 millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.plusMillis(Common.toTimestamp(
            value), Common.toLong(millis)));
   }

   /**
    * Add a number of milliseconds to a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#plusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime plusMillis_ti(DateTime value, Int32 millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.plusMillis(Common.toTimestamp(
            value), Common.toInteger(millis)));
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 minus_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toLong(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 minus_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toLong(a), Common.toInteger(b))
            .longValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 minus_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toInteger(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 minus_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toInteger(a),
                                                           Common.toInteger(b)).longValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double minus_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toBigDecimal(a),
                                                           Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double minus_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toBigDecimal(a),
                                                           Common.toLong(b)).doubleValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double minus_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toBigDecimal(a),
                                                           Common.toInteger(b)).doubleValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double minus_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toLong(a),
                                                           Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Subtract one number from another.
    *
    * @see com.goldencode.p2j.util.MathOps#minus(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double minus_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.minus(Common.toInteger(a),
                                                           Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Calculate a date by subtracting a number of days from a date.
    *
    * @see com.goldencode.p2j.util.date#minusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minus_dl(DateTime value, Int64 days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.minus(Common.toDate(value),
                                                                           Common.toLong(days)));
   }

   /**
    * Calculate a date by subtracting a number of days from a date.
    *
    * @see com.goldencode.p2j.util.date#minusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minus_di(DateTime value, Int32 days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.minus(Common.toDate(value),
                                                                           Common.toInteger(days)));
   }

   /**
    * Calculate a date by subtracting a number of days from a date.
    *
    * @see com.goldencode.p2j.util.date#minusDays(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minus_dr(DateTime value, Double days)
   {
      return Common.fromDate(com.goldencode.p2j.persist.pl.Operators.minus(Common.toDate(value),
                                                                           Common.toBigDecimal(
                                                                                 days)));
   }

   /**
    * Calculate a timestamp by substracting a number of milliseconds from a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#minusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minusMillis_tr(DateTime value, Double millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.minusMillis(Common.toTimestamp(
            value), Common.toBigDecimal(millis)));
   }

   /**
    * Calculate a timestamp by substracting a number of milliseconds from a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#minusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minusMillis_tl(DateTime value, Int64 millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.minusMillis(Common.toTimestamp(
            value), Common.toLong(millis)));
   }

   /**
    * Calculate a timestamp by substracting a number of milliseconds from a datetime.
    *
    * @see com.goldencode.p2j.util.datetime#minusMillis(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.NumberType)
    */
   public static DateTime minusMillis_ti(DateTime value, Int32 millis)
   {
      return Common.fromTimestamp(com.goldencode.p2j.persist.pl.Operators.minusMillis(Common.toTimestamp(
            value), Common.toInteger(millis)));
   }

   /**
    * Determine the number of days between two dates.
    *
    * @see com.goldencode.p2j.util.date#differenceNum(com.goldencode.p2j.util.date,
    * com.goldencode.p2j.util.date)
    */
   public static Int64 dateSpan_dd(DateTime d1, DateTime d2)
   {
      return com.goldencode.p2j.persist.pl.Operators.dateSpan(Common.toDate(d1),
                                                              Common.toDate(d2)).longValue();
   }

   /**
    * Determine the number of milliseconds between two timestamps.
    *
    * @see com.goldencode.p2j.util.datetime#differenceNum(com.goldencode.p2j.util.datetime,
    * com.goldencode.p2j.util.datetime)
    */
   public static Int64 timeSpan_tt(DateTime d1, DateTime d2)
   {
      return com.goldencode.p2j.persist.pl.Operators.timeSpan(Common.toTimestamp(d1),
                                                              Common.toTimestamp(d2)).longValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 multiply_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toLong(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 multiply_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toLong(a),
                                                              Common.toInteger(b)).longValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 multiply_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toInteger(a),
                                                              Common.toLong(b)).longValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Int64 multiply_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toInteger(a),
                                                              Common.toInteger(b)).longValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double multiply_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toBigDecimal(a),
                                                              Common.toBigDecimal(b))
            .doubleValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double multiply_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toBigDecimal(a),
                                                              Common.toLong(b)).doubleValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double multiply_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toBigDecimal(a),
                                                              Common.toInteger(b)).doubleValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double multiply_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toLong(a),
                                                              Common.toBigDecimal(b))
            .doubleValue();
   }

   /**
    * Calculate the product of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#multiply(com.goldencode.p2j.util.int64,
    * com.goldencode.p2j.util.int64)
    */
   public static Double multiply_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.multiply(Common.toInteger(a),
                                                              Common.toBigDecimal(b))
            .doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toLong(a), Common.toLong(b))
            .doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toInteger(a), Common.toLong(b))
            .doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toLong(a), Common.toInteger(b))
            .doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toInteger(a),
                                                            Common.toInteger(b)).doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toBigDecimal(a),
                                                            Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toBigDecimal(a),
                                                            Common.toLong(b)).doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toBigDecimal(a),
                                                            Common.toInteger(b)).doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toLong(a),
                                                            Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Calculate the quotient of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#divide(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Double divide_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.divide(Common.toInteger(a),
                                                            Common.toBigDecimal(b)).doubleValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_ll(Int64 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toLong(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_li(Int64 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toLong(a), Common.toInteger(b))
            .longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_il(Int32 a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toInteger(a), Common.toLong(b))
            .longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_ii(Int32 a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toInteger(a),
                                                            Common.toInteger(b)).longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_rr(Double a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toBigDecimal(a),
                                                            Common.toBigDecimal(b)).longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_rl(Double a, Int64 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toBigDecimal(a),
                                                            Common.toLong(b)).longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_ri(Double a, Int32 b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toBigDecimal(a),
                                                            Common.toInteger(b)).longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_lr(Int64 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toLong(a),
                                                            Common.toBigDecimal(b)).longValue();
   }

   /**
    * Calculate the integer remainder resulting from the integer division of two numbers.
    *
    * @see com.goldencode.p2j.util.MathOps#modulo(com.goldencode.p2j.util.NumberType,
    * com.goldencode.p2j.util.NumberType)
    */
   public static Int64 modulo_ir(Int32 a, Double b)
   {
      return com.goldencode.p2j.persist.pl.Operators.modulo(Common.toInteger(a),
                                                            Common.toBigDecimal(b)).longValue();
   }
}
